(require (lib "test.ss" "schemeunit")
         (lib "text-ui.ss" "schemeunit")
         "let-bindings.ss")

(define bindings '((a . "a") (b . "b1") (b . "b2")))
(define defaults '(("a" . ay) ("b" . bee) (else . cee)))
(define defaults/no-else '(("a" . ay) ("b" . bee)))
(define mapping-fn
  (lambda (value)
    (if value
        (string->symbol value)
        1)))



(define let-bindings-tests
  (make-test-suite
   "let-bindings Test Suite"
   (make-test-case
    "Basic let-bindings variables"
    (assert-equal?
     (list "a" "b2" "c")
     (let-bindings bindings
                   ((a "")
                    (b "")
                    (c "c"))
                   (list a b c))))
   (make-test-case
    "let-bindings without defaults"
    (assert-equal?
     (list "a" "b2")
     (let-bindings bindings
                   ((a) (b))
                   (list a b))))
   (make-test-case
    "single binding with default"
    (assert-equal?
     "a"
     (let-bindings bindings
                   ((a "default"))
                   a)))
   (make-test-case
    "single binding without default"
    (assert-equal?
     "a"
     (let-bindings bindings
                   ((a))
                   a)))
   (make-test-case
    "unbound single binding with default"
    (assert-equal?
     "default"
     (let-bindings bindings
                   ((c "default"))
                   c)))
   (make-test-case
    "unbound single binding without default"
    (assert-exn exn:missing-binding?
                (lambda ()
                  (let-bindings bindings
                                ((c))
                                c))))
   
   (make-test-case
    "single binding with alist default"
    (assert-eq?
     'ay
     (let-bindings bindings
                   ((a defaults))
                   a)))
   (make-test-case
    "multiple bindings with alist defaults"
    (assert-equal?
     (list 'ay 'bee)
     (let-bindings bindings
                   ((a defaults)
                    (b defaults))
                   (list a b))))
   (make-test-case
    "single binding with else default"
    (assert-eq?
     'cee
     (let-bindings bindings
                   ((c defaults))
                   c)))
   (make-test-case
    "single binding with no else default"
    (assert-exn exn:missing-binding?
                (lambda ()
                  (let-bindings bindings
                                ((c defaults/no-else))
                                c))))
   
   ))

(test/text-ui let-bindings-tests)