(module let-bindings mzscheme
  
  (require (lib "cgi.ss" "net")
           (lib "match.ss"))
  (require-for-syntax (lib "stx.ss" "syntax"))
  
  (provide let-bindings
           (struct exn:missing-binding ())
           )
  
  (define-struct (exn:missing-binding exn) ())
  
  (define (raise-missing-binding/default name default)
    (raise (make-exn:missing-binding
            (string-append "No binding found for " 
                           (symbol->string (quote name))
                           " with default " 
                           (format "~a" default))
            (current-continuation-marks))))
  
  (define (get-binding/default name bindings default)
    (cond
      ((pair? default) (get-binding/alist name bindings default))
      ((procedure? default) (get-binding/procedure name bindings default))
      (else (get-binding/value name bindings default))))
  
  
  (define (get-binding/value name bindings value)
    (match (extract-bindings name bindings)
      [() value]
      [(first . _) first]))
  
  (define (get-binding/alist name bindings alist)
    (define (get-default)
      (match (assq 'else alist)
        [#f (raise-missing-binding/default name alist)]
        [(key . value) value]))
    (match (extract-bindings name bindings)
      [() (get-default)]
      [(first . _) (match (assoc first alist)
                     [#f (get-default)]
                     [(key . value) value])]))
  
  (define (get-binding/procedure name bindings proc)
    (match (extract-bindings name bindings)
      [() (proc #f)]
      [(first . _) (proc first)]))
  
  (define-syntax let-bindings
    (lambda (stx)
      (syntax-case stx ()
        ((let-form bindings (var1 var2 ...) expr ...)
         (with-syntax 
             ((the-bindings
               (let loop ((var-first (syntax var1)) (var-rest (syntax (var2 ...))))
                 (let ((binding
                        (syntax-case var-first ()
                          ((name default)
                           (syntax
                            (name (let ((the-bindings (extract-bindings (quote name) bindings)))
                                    (if (null? the-bindings)
                                        (get-binding/default (quote name) bindings default)
                                        (car the-bindings))))))
                          ((name)
                           (syntax
                            (name (let ((the-bindings (extract-bindings (quote name) bindings)))
                                    (if (null? the-bindings)
                                        (raise (make-exn:missing-binding 
                                                (string-append "No binding found for " (symbol->string (quote name)))
                                                (current-continuation-marks)))
                                        (car the-bindings)))))))))
                   (if (stx-null? var-rest)
                       (list binding)
                       (cons binding (loop (stx-car var-rest) (stx-cdr var-rest))))))))
           (syntax (let the-bindings expr ...)))))))
  
  )

